/*
** load_ib.e, based on load_stub.c
**
*/
/*****************************************************************
 *  Title:      load_stub.c
 *  Sccsid:     @(#)load_stub.c	2.1.8.1
 *  Description:
 *              stub routines for:
 *          inline load of dss benchmark
 *          header creation for dss benchmark
 *
 *****************************************************************
 */

#include <stdio.h>
#include "config.h"
#include "dss.h"
#include "dsstypes.h"
#include <time.h>

EXEC SQL
    	SET DATABASE TCSH_GDB = COMPILETIME "tpcr.gdb"
		RUNTIME :db;

void
checksql()
{
	if (SQLCODE < 0)
	{
		isc_print_sqlerror(SQLCODE, isc_status);
		EXEC SQL ROLLBACK;
		EXEC SQL DISCONNECT ALL;
		exit(1);
	}
}

void
makeibdate (char *str, ISC_DATE *ibdate)
{
	struct tm date;
	if (strlen(str) == 8)
	{
		sscanf( str, "%02d-%02d-%02d", &date.tm_year,
			&date.tm_mon, &date.tm_mday );
	} else {
		sscanf( str, "%04d-%02d-%02d", &date.tm_year,
			&date.tm_mon, &date.tm_mday );
		date.tm_year -= 1900;	
	}
	--date.tm_mon;
	date.tm_sec = date.tm_min = date.tm_hour = 0;
	date.tm_wday = date.tm_yday = date.tm_isdst = 0;
	isc_encode_sql_date( (void*)&date, ibdate );
}

int
prep_direct(char *db)
{
    EXEC SQL
    	CONNECT TCSH_GDB;
    checksql();
    
    EXEC SQL
    	SET TRANSACTION READ WRITE WAIT ISOLATION LEVEL SNAPSHOT;
    checksql();

    return(0);
}

int
close_direct(void)
{
	EXEC SQL
		COMMIT;
	EXEC SQL
		DISCONNECT TCSH_GDB;
	return (0);
}

int 
hd_cust (FILE *f)
{
    static int count = 0;

    if (! count++)
        printf("No header has been defined for the customer table\n");

    return(0);
}

int 
ld_cust (customer_t *cp, int mode)
{
	EXEC SQL
		INSERT INTO CUSTOMER (C_CUSTKEY, C_NAME, C_ADDRESS,
		C_NATIONKEY, C_PHONE, C_ACCTBAL, C_MKTSEGMENT,
		C_COMMENT) VALUES (:cp->custkey, :cp->name, :cp->address,
		:cp->nation_code, :cp->phone, :cp->acctbal, :cp->mktsegment,
		:cp->comment);
	
	checksql();
		
    return(0);
}

int 
hd_part (FILE *f)
{
    static int count = 0;

    if (! count++)
        printf("No header has been defined for the part table\n");

    return(0);
}

int 
ld_part (part_t *pp, int mode)
{
	EXEC SQL
		INSERT INTO PART (P_PARTKEY, P_NAME, P_MFGR, P_BRAND,
		P_TYPE, P_SIZE, P_CONTAINER, P_RETAILPRICE, P_COMMENT)
		VALUES (:pp->partkey, :pp->name, :pp->mfgr, :pp->brand,
		:pp->type, :pp->size, :pp->container, :pp->retailprice,
		:pp->comment);

	checksql ();

	return(0);
}

int 
ld_psupp (part_t *pp, int mode)
{
	int i;
	for (i = 0; i < SUPP_PER_PART; i++)
	{
		partsupp_t *ps = &(pp->s[i]);
		
		EXEC SQL
			INSERT INTO PARTSUPP (PS_PARTKEY, PS_SUPPKEY, 
			PS_AVAILQTY, PS_SUPPLYCOST, PS_COMMENT) VALUES 
			(:ps->partkey, :ps->suppkey, :ps->qty, 
			:ps->scost, :ps->comment);
		checksql();
	}	
	return(0);
}


int 
hd_supp (FILE *f)
{
    static int count = 0;

    if (! count++)
        printf("No header has been defined for the supplier table\n");

    return(0);
}

int 
ld_supp (supplier_t *sp, int mode)
{
	EXEC SQL
		INSERT INTO SUPPLIER (S_SUPPKEY, S_NAME, S_ADDRESS,
		S_NATIONKEY, S_PHONE, S_ACCTBAL, S_COMMENT) VALUES
		(:sp->suppkey, :sp->name, :sp->address, :sp->nation_code,
		:sp->phone, :sp->acctbal, :sp->comment);

	checksql();

	return(0);
}


int 
hd_order (FILE *f)
{
    static int count = 0;

    if (! count++)
        printf("No header has been defined for the order table\n");

    return(0);
}

int 
ld_order (order_t *p, int mode)
{
	DSS_HUGE okey = *(p->okey);
	ISC_DATE odate;
	char orderstatus[2];

	makeibdate( p->odate, &odate );
	orderstatus[0] = p->orderstatus;
	orderstatus[1] = 0;
	
	EXEC SQL
		INSERT INTO ORDERS (O_ORDERKEY, O_CUSTKEY, O_ORDERSTATUS,
		O_TOTALPRICE, O_ORDERDATE, O_ORDERPRIORITY, O_CLERK,
		O_SHIPPRIORITY, O_COMMENT) VALUES (:okey,
		:p->custkey, :orderstatus, :p->totalprice,
		:odate, :p->opriority, :p->clerk, :p->spriority,
		:p->comment);
	checksql();
	return(0);
}

int
ld_line (order_t *p, int mode)
{
	int i;

	for (i = 0; i < p->lines; i++)
	{
		line_t *l = &(p->l[i]);
		DSS_HUGE okey = *(l->okey);
		ISC_DATE cdate, sdate, rdate;

		makeibdate( l->cdate, &cdate );
		makeibdate( l->sdate, &sdate );
		makeibdate( l->rdate, &rdate );
		
		EXEC SQL
			INSERT INTO LINEITEM (L_ORDERKEY, L_PARTKEY,
			L_SUPPKEY, L_LINENUMBER, L_QUANTITY,
			L_EXTENDEDPRICE, L_DISCOUNT, L_TAX, L_RETURNFLAG,
			L_LINESTATUS, L_SHIPDATE, L_COMMITDATE,
			L_RECEIPTDATE, L_SHIPINSTRUCT, L_SHIPMODE,
			L_COMMENT) VALUES (:okey, :l->partkey,
			:l->suppkey, :l->lcnt, :l->quantity,
			:l->eprice, :l->discount, :l->tax, :l->rflag,
			:l->lstatus, :sdate, :cdate, :rdate,
			:l->shipinstruct, :l->shipmode, :l->comment);
		checksql();
	}
	return(0);
}



int 
hd_psupp (FILE *f)
{
    static int count = 0;

    if (! count++)
        printf("%s %s\n",
            "No header has been defined for the",
            "part supplier table");

    return(0);
}


int 
hd_line (FILE *f)
{
    static int count = 0;

    if (! count++)
        printf("No header has been defined for the lineitem table\n");

    return(0);
}

int 
hd_nation (FILE *f)
{
    static int count = 0;

    if (! count++)
        printf("No header has been defined for the nation table\n");

    return(0);
}

int 
ld_nation (code_t *cp, int mode)
{
	EXEC SQL
		INSERT INTO NATION (N_NATIONKEY, N_NAME, 
			N_REGIONKEY, N_COMMENT)
		VALUES (:cp->code, :cp->text, :cp->join, :cp->comment);
	checksql();
	return (0);		
}

int 
hd_region (FILE *f)
{
    static int count = 0;

    if (! count++)
        printf("No header has been defined for the region table\n");

    return(0);
}

int 
ld_region (code_t *cp, int mode)
{
	EXEC SQL
		INSERT INTO REGION (R_REGIONKEY, R_NAME, R_COMMENT)
		VALUES (:cp->code, :cp->text, :cp->comment);
	checksql();
	return(0);
}

int 
ld_order_line (order_t *p, int mode)
{
    ld_order(p, mode);
    ld_line (p, mode);

    return(0);
}

int 
hd_order_line (FILE *f)
{
    hd_order(f);
    hd_line (f);

    return(0);
}

int 
ld_part_psupp (part_t *p, int mode)
{
    ld_part(p, mode);
    ld_psupp (p, mode);

    return(0);
}

int 
hd_part_psupp (FILE *f)
{
    hd_part(f);
    hd_psupp(f);

    return(0);
}
