#!/bin/bash

#
# script for running the TPC-R test
#

usage()
{
	echo "usage: runtpc.sh [-h] [-p /path] [-l log] [-s] [query...]"
	echo "       -h ... show help"
	echo "       -p ... prefix for the database (default: /tmp)"
	echo "       -l ... copy log to 'log'"
	echo "       -s ... dont include plan/statistics in the logfile"
	echo "    query ... run query # (1-22)"
	exit 1
}

PREFIX=/tmp
LOG=
RUN=
STAT=Y
while [ -n "$1" ]; do
	if [ "$1" = "-h" ]; then
		usage
	elif [ "$1" = "-p" ]; then
		shift
		if [ -z "$1" ]; then usage; fi
		PREFIX="$1"
	elif [ "$1" = "-l" ]; then
		shift
		if [ -z "$1" ]; then usage; fi
		LOG="$1"
	elif [ "$1" = "-s" ]; then
		STAT=N
	elif [ "$1" -gt 0 -a "$1" -lt 23 ]; then
		RUN="$RUN $1"
	else
		usage
	fi
	shift
done

if [ -z "$RUN" ]; then
	RUN=`seq 1 22`
fi

# create a working file
SCRIPT=`basename "$0"`
SQLTMP=`mktemp /tmp/$SCRIPT.XXXXXX`
ERG=$SQLTMP.txt

# run the tests
if [ "$STAT" = "Y" ]; then
	SETSTATS=`echo -e "{i\\\\\\\\\nset plan;\ni\\\\\\\\\nset stat;\nd\n}"`
else
	SETSTATS="d"
fi

echo "`date` -- TPC-R started" | tee $ERG
for q in $RUN; do
	echo "`date` -- started  $q.sql" | tee -a $ERG
	# remove comments and add the stats command if wanted
	cat queries/$q.sql | \
		sed -e "/^--/d" | \
		sed -e "/^:stat/$SETSTATS" >$SQLTMP
	isql -i $SQLTMP $PREFIX/tpcr.gdb >>$ERG 2>&1
	echo "`date` -- finished $q.sql" | tee -a $ERG
done
echo "`date` -- Test ended" | tee -a $ERG

if [ -n "$LOG" ]; then
	echo "copy log to $LOG" | tee -a $ERG
	cp "$ERG" "$LOG"
fi
	
less $ERG

rm $ERG $SQLTMP
